import { translate } from './i18n.js'

// Possible impacts
// ['ui', 'data', 'limit-bounds', 'datalayer-index', 'remote-data', 'background']

export const SCHEMA = {
  browsable: {
    type: Boolean,
    impacts: ['ui'],
  },
  captionBar: {
    impacts: ['ui'],
    type: Boolean,
    label: translate('Do you want to display a caption bar?'),
    default: false,
    impacts: [],
  },
  captionMenus: {
    impacts: ['ui'],
    type: Boolean,
    label: translate('Do you want to display caption menus?'),
    default: true,
  },
  color: {
    impacts: ['data'],
    type: String,
    handler: 'ColorPicker',
    label: translate('color'),
    helpEntries: 'colorValue',
    inheritable: true,
    default: 'DarkBlue',
  },
  dashArray: {
    impacts: ['data'],
    type: String,
    label: translate('dash array'),
    helpEntries: 'dashArray',
    inheritable: true,
  },
  datalayersControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    handler: 'DataLayersControl',
    label: translate('Display the data layers control'),
    default: true,
  },
  defaultView: {
    impacts: [], // no need to update the ui, only useful when loading the map
    type: String,
    label: translate('Default view'),
    choices: [
      ['center', translate('Saved center and zoom')],
      ['data', translate('Fit all data')],
      ['latest', translate('Latest feature')],
      ['locate', translate('User location')],
    ],
    default: 'center',
  },
  description: {
    label: translate('description'),
    impacts: ['ui'],
    type: 'Text',
    helpEntries: 'textFormatting',
  },
  displayOnLoad: {
    type: Boolean,
    impacts: [],
  },
  displayPopupFooter: {
    type: Boolean,
    impacts: ['ui'],
    label: translate('Do you want to display popup footer?'),
    default: false,
  },
  easing: { impacts: [], type: Boolean, default: false },
  editinosmControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the control to open OpenStreetMap editor'),
    default: null,
  },
  embedControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the embed control'),
    default: true,
  },
  facetKey: { impacts: ['ui'], type: String },
  fill: {
    impacts: ['data'],
    type: Boolean,
    label: translate('fill'),
    helpEntries: 'fill',
    inheritable: true,
    default: true,
  },
  fillColor: {
    impacts: ['data'],
    type: String,
    handler: 'ColorPicker',
    label: translate('fill color'),
    helpEntries: 'fillColor',
    inheritable: true,
  },
  fillOpacity: {
    impacts: ['data'],
    type: Number,
    min: 0.1,
    max: 1,
    step: 0.1,
    label: translate('fill opacity'),
    inheritable: true,
    default: 0.3,
  },
  filterKey: { impacts: [], type: String },
  fromZoom: {
    impacts: [], // not needed
    type: Number,
    label: translate('From zoom'),
    helpText: translate('Optional.'),
  },
  fullscreenControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the fullscreen control'),
    default: true,
  },
  iconClass: {
    impacts: ['data'],
    type: String,
    label: translate('Icon shape'),
    inheritable: true,
    choices: [
      ['Default', translate('Default')],
      ['Circle', translate('Circle')],
      ['Drop', translate('Drop')],
      ['Ball', translate('Ball')],
    ],
    default: 'Default',
  },
  iconOpacity: {
    impacts: ['data'],
    type: Number,
    min: 0.1,
    max: 1,
    step: 0.1,
    label: translate('icon opacity'),
    inheritable: true,
    default: 1,
  },
  iconUrl: {
    impacts: ['data'],
    type: String,
    handler: 'IconUrl',
    label: translate('Icon symbol'),
    inheritable: true,
  },
  inCaption: {
    type: Boolean,
    impacts: ['ui'],
  },

  interactive: {
    impacts: ['data'],
    type: Boolean,
    label: translate('Allow interactions'),
    helpEntries: 'interactive',
    inheritable: true,
    default: true,
  },
  labelDirection: {
    impacts: ['data'],
    type: String,
    label: translate('Label direction'),
    inheritable: true,
    choices: [
      ['auto', translate('Automatic')],
      ['left', translate('On the left')],
      ['right', translate('On the right')],
      ['top', translate('On the top')],
      ['bottom', translate('On the bottom')],
    ],
    default: 'auto',
  },
  labelInteractive: {
    impacts: ['data'],
    type: Boolean,
    label: translate('Labels are clickable'),
    inheritable: true,
  },
  labelKey: {
    impacts: ['data'],
    type: String,
    helpEntries: 'labelKey',
    placeholder: translate('Default: name'),
    label: translate('Label key'),
    inheritable: true,
  },
  licence: { impacts: ['ui'], type: String, label: translate('licence') },
  limitBounds: { impacts: ['limit-bounds'], type: Object },
  locateControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the locate control'),
  },
  longCredit: {
    impacts: ['ui'],
    type: 'Text',
    label: translate('Long credits'),
    helpEntries: ['longCredit', 'textFormatting'],
  },
  measureControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the measure control'),
  },
  miniMap: {
    impacts: ['ui'],
    type: Boolean,
    label: translate('Do you want to display a minimap?'),
    default: false,
  },
  moreControl: {
    impacts: ['ui'],
    type: Boolean,
    label: translate('Do you want to display the «more» control?'),
    default: true,
  },
  name: {
    impacts: ['ui', 'data'],
    type: String,
    label: translate('name'),
  },
  onLoadPanel: {
    impacts: [], // This is what happens during the map instantiation
    type: String,
    label: translate('Do you want to display a panel on load?'),
    choices: [
      ['none', translate('None')],
      ['caption', translate('Caption')],
      ['databrowser', translate('Data browser')],
      ['facet', translate('Facet search')],
    ],
    default: 'none',
  },
  opacity: {
    impacts: ['data'],
    type: Number,
    min: 0.1,
    max: 1,
    step: 0.1,
    label: translate('opacity'),
    inheritable: true,
    default: 0.5,
  },
  outlink: {
    type: String,
    label: translate('Link to…'),
    helpEntries: 'outlink',
    placeholder: 'http://...',
    inheritable: true,
  },
  outlinkTarget: {
    impacts: [],
    type: String,
    label: translate('Open link in…'),
    inheritable: true,
    default: 'blank',
    choices: [
      ['blank', translate('new window')],
      ['self', translate('iframe')],
      ['parent', translate('parent window')],
    ],
  },
  overlay: { impacts: ['background'], type: Object },
  permanentCredit: {
    impacts: ['ui'],
    type: 'Text',
    label: translate('Permanent credits'),
    helpEntries: ['permanentCredit', 'textFormatting'],
  },
  permanentCreditBackground: {
    impacts: ['ui'],
    type: Boolean,
    label: translate('Permanent credits background'),
    default: true,
  },
  popupContentTemplate: {
    impacts: [], // not needed
    type: 'Text',
    label: translate('Popup content template'),
    helpEntries: ['dynamicProperties', 'textFormatting'],
    placeholder: '# {name}',
    inheritable: true,
    default: '# {name}\n{description}',
  },
  popupShape: {
    impacts: [], // not needed
    type: String,
    label: translate('Popup shape'),
    inheritable: true,
    choices: [
      ['Default', translate('Popup')],
      ['Large', translate('Popup (large)')],
      ['Panel', translate('Side panel')],
    ],
    default: 'Default',
  },
  popupTemplate: {
    impacts: [], // not needed
    type: String,
    label: translate('Popup content style'),
    inheritable: true,
    choices: [
      ['Default', translate('Default')],
      ['Table', translate('Table')],
      ['GeoRSSImage', translate('GeoRSS (title + image)')],
      ['GeoRSSLink', translate('GeoRSS (only link)')],
      ['OSM', translate('OpenStreetMap')],
    ],
    default: 'Default',
  },
  remoteData: {
    type: Object,
    impacts: ['remote-data'],
  },
  scaleControl: {
    impacts: ['ui'],
    type: Boolean,
    label: translate('Do you want to display the scale control?'),
    default: true,
  },
  scrollWheelZoom: {
    impacts: ['ui'],
    type: Boolean,
    label: translate('Allow scroll wheel zoom?'),
  },
  searchControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the search control'),
    default: true,
  },
  shortCredit: {
    impacts: ['ui'],
    type: String,
    label: translate('Short credits'),
    helpEntries: ['shortCredit', 'textFormatting'],
  },
  showLabel: {
    impacts: ['data'],
    type: Boolean,
    nullable: true,
    label: translate('Display label'),
    inheritable: true,
    default: false,
  },
  slideshow: { impacts: ['ui'], type: Object },
  slugKey: { impacts: [], type: String },
  smoothFactor: {
    impacts: ['data'],
    type: Number,
    min: 0,
    max: 10,
    step: 0.5,
    label: translate('Simplify'),
    helpEntries: 'smoothFactor',
    inheritable: true,
    default: 1.0,
  },
  sortKey: { impacts: ['data', 'datalayer-index'], type: String },
  starControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the star map button'),
  },
  stroke: {
    impacts: ['data'],
    type: Boolean,
    label: translate('stroke'),
    helpEntries: 'stroke',
    inheritable: true,
    default: true,
  },
  tilelayer: { impacts: ['background'], type: Object },
  tilelayersControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the tile layers control'),
  },
  toZoom: {
    type: Number,
    impacts: [], // not needed
    label: translate('To zoom'),
    helpText: translate('Optional.'),
  },
  type: {
    type: 'String',
    impacts: ['data'],
  },
  weight: {
    impacts: ['data'],
    type: Number,
    min: 1,
    max: 20,
    step: 1,
    label: translate('weight'),
    inheritable: true,
    default: 3,
  },
  zoom: {
    impacts: [], // default zoom, doesn't need to be updated
    type: Number,
  },
  zoomControl: {
    impacts: ['ui'],
    type: Boolean,
    nullable: true,
    label: translate('Display the zoom control'),
    default: true,
  },
  zoomTo: {
    impacts: [], // not need to update the view
    type: Number,
    placeholder: translate('Inherit'),
    helpEntries: 'zoomTo',
    label: translate('Default zoom level'),
    inheritable: true,
  },
}
